package org.netkernel.layer0.meta.impl;

import java.io.ByteArrayInputStream;
import java.util.regex.*;
import javax.xml.parsers.ParserConfigurationException;
import org.netkernel.container.IMessages;
import org.netkernel.layer0.bnf.BNFGrammarFactory;
import org.netkernel.layer0.bnf.GrammarParseException;
import org.netkernel.layer0.meta.IIdentifierGrammar;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.util.XMLUtils;
import org.netkernel.urii.INetKernelException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

// TODO: validation, message bundle errors, optimised use of regex, declare all metadata arguments, build grammar directly not an XML string to be parsed
public class SimpleGrammarParser implements IGrammarParser
{
	private static final Pattern mPathPattern=Pattern.compile("\\{(.*?)\\}", Pattern.DOTALL);
	private static final String REPLACEMENT_OPEN_BRACKET="RePlAcEoPeN";
	private static final String REPLACEMENT_CLOSE_BRACKET="RePlAcEcLoSe";
	
	public void parseGrammar(Element aGrammarDefn, EndpointMetaBuilder aBuilder, IMessages aMessages) throws ParserConfigurationException, GrammarParseException, INetKernelException
	{
		String grammar=XMLUtils.getText(aGrammarDefn);
		grammar=grammar.replaceAll("\\x5C\\x7B", REPLACEMENT_OPEN_BRACKET);
		grammar=grammar.replaceAll("\\x5C\\x7D", REPLACEMENT_CLOSE_BRACKET);
		Matcher matcher=mPathPattern.matcher(grammar);
		int start=0;
		try
		{	while(matcher.find(start))
			{	String a=matcher.group(1);
				String regex;
				int split=a.indexOf(":");
				if(split>0)
				{	regex=a.substring(split+1);
					a=a.substring(0,split);
				}
				else
				{	regex=".*?";				
				}
				a=XMLUtils.escape(a);
				regex=XMLUtils.escape(regex);
				String sub="<group name=\""+a+"\"><regex>"+regex+"</regex></group>";
				start=matcher.start();
				grammar=matcher.replaceFirst(sub);
				matcher.reset(grammar);
				start+=sub.length();
			}
		}
		catch(IndexOutOfBoundsException e)
		{	throw new NKFException("SimpleGrammarException", "Bad grammar: "+grammar);
		}
		grammar="<grammar>"+grammar+"</grammar>";
		grammar=grammar.replaceAll(REPLACEMENT_OPEN_BRACKET, "{");
		grammar=grammar.replaceAll(REPLACEMENT_CLOSE_BRACKET, "}");
		try
		{	Document d=XMLUtils.parse(new ByteArrayInputStream(grammar.toString().getBytes()));
			IIdentifierGrammar theGrammar=BNFGrammarFactory.parse(d.getDocumentElement(), aMessages);
			aBuilder.setGrammar(theGrammar);
		}
		catch (Exception e)
		{	throw new NKFException("Unexpected Exception",null,e);
		}
	}

}
