/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf.impl;

import org.netkernel.container.IKernel;
import org.netkernel.layer0.nkf.*;
import org.netkernel.urii.IEndpointLifecycle;
import org.netkernel.urii.ISpace;

/**
 * Base class for developing NetKernel Transports on the NKF API
 * @author  tab
 */
public abstract class NKFTransportImpl implements IEndpointLifecycle
{
	private IKernel mKernel;
	private ISpace mSpace;
	private NKFContextImpl mContext;
	
	/**
	 * Low-level transport start, performs NKF-specific inititialisation
	 */
	public final void onCommissionEndpoint(IKernel aKernel, ISpace aSpace) throws Exception
	{	mKernel=aKernel;
		mSpace=aSpace;
		mContext = new NKFTransportContextImpl(mKernel,mSpace,this);
			
		INKFRequestContext initContext = new NKFTransportInitContextImpl(mKernel,mSpace,this);
		onCommissionTransport(initContext);
	}
	
	protected abstract void onCommissionTransport(INKFRequestContext aContext) throws Exception;
	
	/**
	 * Low-level transport stop, performs NKF-speocific shutdown
	 */
	public final void onDecommissionEndpoint() throws Exception
	{	onDecommissionTransport();
	}
	
	protected abstract void onDecommissionTransport() throws Exception;

	/**
	 *  Get an INKFRequestContext context object through which to interact with the Kernel.
	 * 
	 */
	protected final INKFRequestContext getContext()
	{	return mContext;
	}
	
}