package org.netkernel.layer0.representation.impl;
import java.util.*;

import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;

public class HDSBuilder
{
	HDSNodeImpl mRoot=new HDSNodeImpl(null,null);
	HDSNodeImpl mCursor=mRoot;
	List mChildren=new ArrayList();
	int mCardinality=0;
	
	
	/**
	 * Construct a new HDSBuilder starting with an empty root;
	 */
	public HDSBuilder()
	{	mRoot=new HDSNodeImpl(null,null);
		mCursor=mRoot;
		mChildren=new ArrayList();
	}
	
	/** Import the given node into the structure at the cursor.
	 *  Cursor will be unmoved.
	 * @param aNode
	 */
	public void importNode(IHDSNode aNode)
	{	pushNode(aNode.getName(),aNode.getValue());
		for (IHDSNode child: aNode.getChildren())
		{	importNode(child);
		}
		popNode();
	}
	
	public void importChildren(IHDSNode aNode)
	{	IHDSNode[] children = aNode.getChildren();
		for (IHDSNode child: children)
		{	importNode(child);
		}
	}
	
	
	public IHDSNode getRoot()
	{	flushChildren();
		return mRoot;
	}
	
	public void addNode(String aName, Object aValue)
	{	mChildren.add(new HDSNodeImpl(aName,aValue,mCursor));
	}
	
	public void pushNode(String aName)
	{	pushNode(aName,null);
	}
	
	public void pushNode(String aName, Object aValue)
	{	HDSNodeImpl node = new HDSNodeImpl(aName,aValue,mCursor);
		mChildren.add(node);
		flushChildren();
		mCursor=node;
		mCardinality++;
	}
	
	public void popNode()
	{	if (mCursor!=mRoot)
		{	flushChildren();
			mCursor=(HDSNodeImpl)mCursor.getParent();
			mCardinality--;
		}
		else
		{	throw new IllegalStateException();
		}
	}
	
	public void setValue(Object aValue)
	{	mCursor.setValue(aValue);
	}
	
	public void deleteNode()
	{
		if (mCursor!=mRoot)
		{	HDSNodeImpl parent=(HDSNodeImpl)mCursor.getParent();
			mChildren.clear();
			for (IHDSNode child: parent.getChildren())
			{	if (child!=mCursor)
				{	mChildren.add(child);
				}
			}
			mCursor=parent;
			mCardinality--;
			mCursor.setChildren(mChildren);
			mChildren.clear();
		}
		else
		{	throw new IllegalStateException();
		}
	}
		
	public void setCursor(IHDSNode aNode)
	{
		flushChildren();
		if (aNode.getRoot()!=mRoot)
		{	throw new IllegalArgumentException();
		}
		mCursor=(HDSNodeImpl)aNode;
		int depth=0;
		IHDSNode c=mCursor;
		while((c=c.getParent())!=null)
		{	depth++;
		}
		mCardinality=depth;
	}
	
	public IHDSNode getCursor()
	{	flushChildren();
		return mCursor;
	}
	
	public int getCursorDepth()
	{	return mCardinality;		
	}
	
	private void flushChildren()
	{	if (mChildren.size()>0)
		{	IHDSNode[] existingChildren=mCursor.getChildren();
			IHDSNode[] newChildren = new IHDSNode[mChildren.size()+existingChildren.length];
			System.arraycopy(existingChildren, 0, newChildren, 0, existingChildren.length);
			int j=existingChildren.length;
			for (Iterator i=mChildren.iterator(); i.hasNext(); )
			{	newChildren[j++]=(IHDSNode)i.next();
			}
			mCursor.setChildren(newChildren);
		}
		mChildren.clear();
	}
	
}
